#!/usr/bin/perl -w
#
# help/resources/autoinstall.pl
# (C) Copyright 2009 Amberdms Ltd <jethro.carr@amberdms.com>
# This utility is licensed under the GNU AGPL Version 3.0
#
# This script is run manually by the user to setup the MySQL database
# for the Amberdms Billing system. This setup is for a single instance
# installation, but it can be cloned for a multi-instance installation
# if desired, as per the documentation.
#
# * Create the MySQL database
# * Create user in MySQL
# * Write settings to config file.
# * Import the inital database.
#
#

use strict;
use DBI;

## SETTINGS ##

# default settings 
# (only need to change these if you are doing development work)
my $db_user		= "root";		# name of user to be used to create data
my $db_name		= "billing_system";	# name of the DB to create
my $db_host		= "localhost";		# MySQL server

my $db_bs_user		= "billing_system";	# name of the MySQL user to create
my $db_bs_password	= random_password(10);	# random password to generate

# location of config.php file
my $opt_cfgfile = "/etc/amberdms/billing_system/config.php";

# tmp file to use for SQL query generation
my $opt_tmpfile = "/tmp/billing_system_mysqlquery";

# location of install schema
my $opt_schemadata;
if ($0 =~ /^([\S\s]*)\/resources\/autoinstall.pl$/)
{
	$opt_schemadata = "$1/schema/";
}
else
{
	$opt_schemadata = "../schema/";
}


## CHECKS ##

# make sure config file is installed
if (! -e $opt_cfgfile)
{
	die("Error: $opt_cfgfile does not exist\n");
}



## PROGRAM ##

system("touch $opt_tmpfile");
system("chown root:root $opt_tmpfile");
system("chmod 600 $opt_tmpfile");


print "autoinstall.pl\n";
print "\n";
print "This script setups the Amberdms Billing System database components:\n";
print " * Amberdms Billing System MySQL user\n";
print " * Amberdms Billing System database\n";
print " * Amberdms Billing System configuration files\n";
print "\n";
print "THIS SCRIPT ONLY NEEDS TO BE RUN FOR THE VERY FIRST INSTALL OF ABS.\n";
print "DO NOT RUN FOR ANY OTHER REASON\n";
print "\n";

print "Please enter MySQL $db_user password (if any): ";
my $db_pass = get_question('^[\S\s]*$');


## 0. CREATE THE DATABASE

# Firstly, connect to the DB and create an empty database for the import
# to take place into.

my $mysql_handle = DBI->connect("dbi:mysql:host=$db_host;user=$db_user;password=$db_pass") || die("Error: Unable to connect to MySQL database: $DBI::errstr\n");

my $mysql_string = "CREATE DATABASE `$db_name` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci";
my $mysql_result = $mysql_handle->prepare($mysql_string) || die("Error: SQL query ($mysql_string) failed: $DBI::errstr\n");
$mysql_result->execute || die("Error: SQL query ($mysql_string) failed: $DBI::errstr\n");


# Re-connect into the DB itself.
$mysql_handle = DBI->connect("dbi:mysql:database=$db_name;host=$db_host;user=$db_user;password=$db_pass") || die("Error: Unable to connect to MySQL database: $DBI::errstr\n");



## 1. IMPORT INSTALL SCHEMA

# import the latest database
print "Searching $opt_schemadata for latest install schema...\n";

# determine the latest install file
my @data	= glob("$opt_schemadata/version_*_install.sql");
my $count	= scalar @data;

if ($count == 0)
{
	die("Error: No schema found in $opt_schemadata!\n");
}
else
{
	$count = $count - 1;
	
	print $data[$count] ." is the latest file and will be used for the install.\n";
	# import schema
	print "Importing file $data[$count]\n";
	import_sql($data[$count], $mysql_handle);
}


## 2. IMPORT UPGRADES

# We only generate new install schemas when a number of changes have been made.
# For minor stuff, we just create an upgrade schemas, which we apply ontop of the
# most recent install schema file.

print "Searching $opt_schemadata for latest upgrade schemas...\n";

# get a list of upgrade files
@data	= glob("$opt_schemadata/version_*_upgrade.sql");
$count	= scalar @data;

if ($count == 0)
{
	print "Installed schema is the latest, no upgrades to apply.\n";
}
else
{
	print "Applying any upgrade SQL files that match...\n";
		
	my ($latestversion, $opt_version_schema);

	$mysql_string = "SELECT value FROM `config` WHERE name='SCHEMA_VERSION'";
	$mysql_result = $mysql_handle->prepare($mysql_string) || die("Error: SQL query ($mysql_string) failed: $DBI::errstr\n");
	$mysql_result->execute;

	while (my $mysql_data = $mysql_result->fetchrow_hashref())
	{
		$opt_version_schema = $mysql_data->{value};
	}
	$mysql_result->finish;


	foreach my $sqlfile (@data)
	{
		if ($sqlfile =~ /_([0-9]{4}[0-9]{2}[0-9]{2}[0-9]*)_/)
		{
			$latestversion = $1;
				
			if ($latestversion > $opt_version_schema)
			{
				# Need to import this schema upgrade file
				print "Applying DB update $sqlfile\n";
				import_sql($sqlfile, $mysql_handle);

				# update the schema version in the DB
				$mysql_string = "UPDATE `config` SET value='$latestversion' WHERE name='SCHEMA_VERSION' LIMIT 1";
				$mysql_result = $mysql_handle->prepare($mysql_string) || die("Error: SQL query ($mysql_string) failed: $DBI::errstr\n");
				$mysql_result->execute;
			}
		}
		else
		{
			print "Warning: Incorrectly named SQL upgrade file: $sqlfile\n";
		}
	}
}



## 3. SETUP MYSQL USER

# create queries in tmp file
open (MYSQL, ">$opt_tmpfile") || die("Unable to create tmp file mysqltmp\n");

# create MySQL user
print "Creating user...\n";

if ($db_host eq "localhost")
{
	print MYSQL "GRANT USAGE ON * . * TO '$db_bs_user'\@'localhost' IDENTIFIED BY '$db_bs_password' WITH MAX_QUERIES_PER_HOUR 0 MAX_CONNECTIONS_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 ;\n";
	print MYSQL "GRANT SELECT , INSERT , UPDATE , DELETE , CREATE , DROP , INDEX , ALTER , CREATE TEMPORARY TABLES, LOCK TABLES ON `$db_name` . * TO '$db_bs_user'\@'localhost';\n";
}
else
{
	print MYSQL "GRANT USAGE ON * . * TO '$db_bs_user'\@'%' IDENTIFIED BY '$db_bs_password' WITH MAX_QUERIES_PER_HOUR 0 MAX_CONNECTIONS_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 ;\n";
	print MYSQL "GRANT SELECT , INSERT , UPDATE , DELETE , CREATE , DROP , INDEX , ALTER , CREATE TEMPORARY TABLES, LOCK TABLES ON `$db_name` . * TO '$db_bs_user'\@'%';\n";
}

close(MYSQL);

# run queries
import_sql($opt_tmpfile, $mysql_handle);

# remove tmp file.
system("rm -f $opt_tmpfile");



## 4. WRITE CONFIGURATION FILE

# update configuration file
print "Updating configuration file...\n";

open (CFG, $opt_cfgfile) || die("Unable to open config file to update settings.");
open (CFGTMP, ">$opt_cfgfile.new") || die("Unable to create tmp config file.");

while (my $line = <CFG>)
{
	chomp($line);

	# update config file with mysql user settings
	if ($line =~ /^\$config\["db_host"\]\s*=\s*"(\S*)";/)
	{
		$line =~ s/"$1"/"$db_host"/;
	}
	
	if ($line =~ /^\$config\["db_name"\]\s*=\s*"(\S*)";/)
	{
		$line =~ s/"$1"/"$db_name"/;
	}
		
	if ($line =~ /^\$config\["db_user"\]\s*=\s*"(\S*)";/)
	{
		$line =~ s/"$1"/"$db_bs_user"/;
	}
	
	if ($line =~ /^\$config\["db_pass"\]\s*=\s*"(\S*)";/)
	{
		$line =~ s/"$1"/"$db_bs_password"/;
	}

	print CFGTMP "$line\n";
}
close(CFG);
close(CFGTMP);

# we use cat instead of mv to retain permissions
system("cat $opt_cfgfile.new > $opt_cfgfile");
system("rm -f $opt_cfgfile.new");



print "DB installation complete!\n";
$mysql_handle->disconnect;
exit 0;


### FUNCTIONS ###

# get_question
#
# Gets user input from CLI and does verification
#
sub get_question {
        my $regex = shift;
        my $complete = 0;


        while (!$complete)
        {
                my $input = <STDIN>;
                chomp($input);

                if ($input =~ /$regex/)
                {
                        return $input;
                }
                else
                {
                        print "Invalid input! Please re-enter.\n";
			print "Retry: ";
                }
        }

}

# random_password
#
# generates a random password of any desired length
#
sub random_password
{
	my $password;
	my $_rand;

	my $password_length = $_[0];

	if (!$password_length) {
		$password_length = 10;
	}

	my @chars = split(" ",
		"a b c d e f g h i j k l m n o
		p q r s t u v w x y z
		0 1 2 3 4 5 6 7 8 9");

	srand;

	for (my $i=0; $i <= $password_length ;$i++)
	{
		$_rand = int(rand (scalar @chars));
		$password .= $chars[$_rand];
	}
	return $password;
}


# import_sql ( filename, MySQL handle )
#
# Imports the specified SQL file into MySQL
#
sub import_sql
{
	my $sqlfile		= shift;
	my $mysql_handle	= shift;

	open(SQL, "$sqlfile") or die("Error: Unable to open $sqlfile\n");


	# Filter out unwanted content
	my @lines;
	while (<SQL>)
	{
		# remove crap lines
		if ($_ =~ /^\s*$/)
		{
			next;
		}
		
		if ($_ =~ /^#/)
		{
			next;
		}

		# Ignore any CREATE DATABASE/USE lines. These exist in older install SQL
		# files but mess with our smarter installers.
		if ($_ =~ /^CREATE\sDATABASE/)
		{
			next;
		}

		if ($_ =~ /^USE/)
		{
			next;
		}

		# Valid SQL line to process
		push (@lines, $_);
	}

				
	# A SQL statement can be multiline, need to rejoin into proper queries
	my @statements = split(/;\n/,join('',@lines));
	foreach my $sqlline ( @statements )
	{
		$mysql_handle->do($sqlline);
	}
				    
	close(SQL);
}

